# coding: utf-8

# flake8: noqa

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.auth.client.api.admin_api import AdminApi
from crypticorn.auth.client.api.auth_api import AuthApi
from crypticorn.auth.client.api.service_api import ServiceApi
from crypticorn.auth.client.api.user_api import UserApi
from crypticorn.auth.client.api.wallet_api import WalletApi

# import ApiClient
from crypticorn.auth.client.api_response import ApiResponse
from crypticorn.auth.client.api_client import ApiClient
from crypticorn.auth.client.configuration import Configuration
from crypticorn.auth.client.exceptions import OpenApiException
from crypticorn.auth.client.exceptions import ApiTypeError
from crypticorn.auth.client.exceptions import ApiValueError
from crypticorn.auth.client.exceptions import ApiKeyError
from crypticorn.auth.client.exceptions import ApiAttributeError
from crypticorn.auth.client.exceptions import ApiException

# import models into sdk package
from crypticorn.auth.client.models.add_wallet200_response import AddWallet200Response
from crypticorn.auth.client.models.add_wallet_request import AddWalletRequest
from crypticorn.auth.client.models.authorize_user200_response import (
    AuthorizeUser200Response,
)
from crypticorn.auth.client.models.authorize_user200_response_auth import (
    AuthorizeUser200ResponseAuth,
)
from crypticorn.auth.client.models.authorize_user_request import AuthorizeUserRequest
from crypticorn.auth.client.models.create_api_key200_response import (
    CreateApiKey200Response,
)
from crypticorn.auth.client.models.create_api_key_request import CreateApiKeyRequest
from crypticorn.auth.client.models.create_user_request import CreateUserRequest
from crypticorn.auth.client.models.get_api_keys200_response_inner import (
    GetApiKeys200ResponseInner,
)
from crypticorn.auth.client.models.list_wallets200_response import (
    ListWallets200Response,
)
from crypticorn.auth.client.models.list_wallets200_response_balances_inner import (
    ListWallets200ResponseBalancesInner,
)
from crypticorn.auth.client.models.list_wallets200_response_balances_inner_sale_round import (
    ListWallets200ResponseBalancesInnerSaleRound,
)
from crypticorn.auth.client.models.list_wallets200_response_balances_inner_wallet import (
    ListWallets200ResponseBalancesInnerWallet,
)
from crypticorn.auth.client.models.list_wallets200_response_balances_inner_wallet_vesting_wallets_inner import (
    ListWallets200ResponseBalancesInnerWalletVestingWalletsInner,
)
from crypticorn.auth.client.models.list_wallets200_response_data_inner import (
    ListWallets200ResponseDataInner,
)
from crypticorn.auth.client.models.list_wallets200_response_user_value import (
    ListWallets200ResponseUserValue,
)
from crypticorn.auth.client.models.logout_default_response import LogoutDefaultResponse
from crypticorn.auth.client.models.logout_default_response_issues_inner import (
    LogoutDefaultResponseIssuesInner,
)
from crypticorn.auth.client.models.oauth_callback200_response import (
    OauthCallback200Response,
)
from crypticorn.auth.client.models.oauth_callback200_response_user import (
    OauthCallback200ResponseUser,
)
from crypticorn.auth.client.models.refresh_token_info200_response import (
    RefreshTokenInfo200Response,
)
from crypticorn.auth.client.models.refresh_token_info200_response_user_session import (
    RefreshTokenInfo200ResponseUserSession,
)
from crypticorn.auth.client.models.resend_verification_email_request import (
    ResendVerificationEmailRequest,
)
from crypticorn.auth.client.models.revoke_user_tokens_request import (
    RevokeUserTokensRequest,
)
from crypticorn.auth.client.models.rotate_tokens200_response import (
    RotateTokens200Response,
)
from crypticorn.auth.client.models.token_info200_response import TokenInfo200Response
from crypticorn.auth.client.models.unlink_wallet_request import UnlinkWalletRequest
from crypticorn.auth.client.models.update_user_request import UpdateUserRequest
from crypticorn.auth.client.models.user_by_username200_response import (
    UserByUsername200Response,
)
from crypticorn.auth.client.models.user_reset_password_request import (
    UserResetPasswordRequest,
)
from crypticorn.auth.client.models.user_set_password_request import (
    UserSetPasswordRequest,
)
from crypticorn.auth.client.models.verify200_response import Verify200Response
from crypticorn.auth.client.models.verify_email200_response import (
    VerifyEmail200Response,
)
from crypticorn.auth.client.models.verify_email200_response_auth import (
    VerifyEmail200ResponseAuth,
)
from crypticorn.auth.client.models.verify_email200_response_auth_auth import (
    VerifyEmail200ResponseAuthAuth,
)
from crypticorn.auth.client.models.verify_email_request import VerifyEmailRequest
from crypticorn.auth.client.models.verify_wallet_request import VerifyWalletRequest
from crypticorn.auth.client.models.wallet_verified200_response import (
    WalletVerified200Response,
)
from crypticorn.auth.client.models.whoami200_response import Whoami200Response
