# coding: utf-8

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt
from typing import Any, ClassVar, Dict, List, Union
from crypticorn.auth.client.models.list_wallets200_response_balances_inner import (
    ListWallets200ResponseBalancesInner,
)
from crypticorn.auth.client.models.list_wallets200_response_data_inner import (
    ListWallets200ResponseDataInner,
)
from crypticorn.auth.client.models.list_wallets200_response_user_value import (
    ListWallets200ResponseUserValue,
)
from typing import Optional, Set
from typing_extensions import Self


class ListWallets200Response(BaseModel):
    """
    ListWallets200Response
    """  # noqa: E501

    data: List[ListWallets200ResponseDataInner]
    count: Union[StrictFloat, StrictInt]
    balances: List[ListWallets200ResponseBalancesInner]
    user_value: ListWallets200ResponseUserValue = Field(alias="userValue")
    __properties: ClassVar[List[str]] = ["data", "count", "balances", "userValue"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ListWallets200Response from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data (list)
        _items = []
        if self.data:
            for _item_data in self.data:
                if _item_data:
                    _items.append(_item_data.to_dict())
            _dict["data"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in balances (list)
        _items = []
        if self.balances:
            for _item_balances in self.balances:
                if _item_balances:
                    _items.append(_item_balances.to_dict())
            _dict["balances"] = _items
        # override the default output from pydantic by calling `to_dict()` of user_value
        if self.user_value:
            _dict["userValue"] = self.user_value.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ListWallets200Response from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "data": (
                    [
                        ListWallets200ResponseDataInner.from_dict(_item)
                        for _item in obj["data"]
                    ]
                    if obj.get("data") is not None
                    else None
                ),
                "count": obj.get("count"),
                "balances": (
                    [
                        ListWallets200ResponseBalancesInner.from_dict(_item)
                        for _item in obj["balances"]
                    ]
                    if obj.get("balances") is not None
                    else None
                ),
                "userValue": (
                    ListWallets200ResponseUserValue.from_dict(obj["userValue"])
                    if obj.get("userValue") is not None
                    else None
                ),
            }
        )
        return _obj
