# coding: utf-8

# flake8: noqa

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.trade.client.api.api_keys_api import APIKeysApi
from crypticorn.trade.client.api.admin_api import AdminApi
from crypticorn.trade.client.api.bots_api import BotsApi
from crypticorn.trade.client.api.exchanges_api import ExchangesApi
from crypticorn.trade.client.api.futures_trading_panel_api import FuturesTradingPanelApi
from crypticorn.trade.client.api.notifications_api import NotificationsApi
from crypticorn.trade.client.api.orders_api import OrdersApi
from crypticorn.trade.client.api.status_api import StatusApi
from crypticorn.trade.client.api.strategies_api import StrategiesApi
from crypticorn.trade.client.api.trading_actions_api import TradingActionsApi

# import ApiClient
from crypticorn.trade.client.api_response import ApiResponse
from crypticorn.trade.client.api_client import ApiClient
from crypticorn.trade.client.configuration import Configuration
from crypticorn.trade.client.exceptions import OpenApiException
from crypticorn.trade.client.exceptions import ApiTypeError
from crypticorn.trade.client.exceptions import ApiValueError
from crypticorn.trade.client.exceptions import ApiKeyError
from crypticorn.trade.client.exceptions import ApiAttributeError
from crypticorn.trade.client.exceptions import ApiException

# import models into sdk package
from crypticorn.trade.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.trade.client.models.api_error_level import ApiErrorLevel
from crypticorn.trade.client.models.api_error_type import ApiErrorType
from crypticorn.trade.client.models.bot import Bot
from crypticorn.trade.client.models.bot_create import BotCreate
from crypticorn.trade.client.models.bot_status import BotStatus
from crypticorn.trade.client.models.bot_update import BotUpdate
from crypticorn.trade.client.models.exception_detail import ExceptionDetail
from crypticorn.trade.client.models.exchange import Exchange
from crypticorn.trade.client.models.exchange_key import ExchangeKey
from crypticorn.trade.client.models.exchange_key_create import ExchangeKeyCreate
from crypticorn.trade.client.models.exchange_key_update import ExchangeKeyUpdate
from crypticorn.trade.client.models.execution_ids import ExecutionIds
from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.futures_trading_action import FuturesTradingAction
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.log_level import LogLevel
from crypticorn.trade.client.models.margin_mode import MarginMode
from crypticorn.trade.client.models.market_type import MarketType
from crypticorn.trade.client.models.notification import Notification
from crypticorn.trade.client.models.notification_create import NotificationCreate
from crypticorn.trade.client.models.notification_update import NotificationUpdate
from crypticorn.trade.client.models.order import Order
from crypticorn.trade.client.models.order_status import OrderStatus
from crypticorn.trade.client.models.post_futures_action import PostFuturesAction
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_exchange_info import StrategyExchangeInfo
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.models.tpsl import TPSL
from crypticorn.trade.client.models.trading_action_type import TradingActionType
